pub enum SmtObligation {
Aligned {
place: PlaceKey,
align: u64,
ty_name: String,
},
NonZero {
place: PlaceKey,
},
Range {
value: PlaceKey,
lower: i128,
upper: Option<i128>,
},
InBounds {
place: PlaceKey,
ty_name: String,
elem_size: u64,
access_count: SmtTerm,
},
PointerRangeInBounds {
place: PlaceKey,
ty_name: String,
lower_delta: SmtTerm,
upper_delta: SmtTerm,
},
Initialized {
place: PlaceKey,
ty_name: String,
elements: SmtTerm,
elem_size: Option<u64>,
array_elem_size: Option<u64>,
array_len_term: Option<SmtTerm>,
},
Allocated {
place: PlaceKey,
ty_name: String,
elements: SmtTerm,
},
NonOverlapping {
left: PlaceKey,
right: PlaceKey,
left_count: SmtTerm,
right_count: SmtTerm,
elem_size: u64,
},
Predicate {
predicates: Vec<SmtPredicate>,
},
}Expand description
General SMT obligation produced by an SP-specific lowering.
Variants
Aligned
Prove that the address denoted by place is aligned to align bytes.
NonZero
Future form for non-zero integer/address requirements.
Range
Future form for interval/bounds requirements.
InBounds
Prove that place points to access_count elements inside its object.
PointerRangeInBounds
Prove that pointer arithmetic starting at place stays inside the
matched object for the whole delta range.
Initialized
Prove that place denotes initialized memory for elements elements.
Fields
Allocated
Prove that place points to elements elements in one live allocation.
NonOverlapping
Prove that two pointer ranges do not overlap.
Predicate
Prove one or more numeric predicates.
Fields

predicates: Vec<SmtPredicate>Implementations